@@ -154,7 +154,8 @@ UV next_trial_prime(UV n)
 
   d = n/30;
   m = n - d*30;
-  m = nextwheel30[m];  if (m == 1) d++;
+  /* Move forward one, knowing we may not be on the wheel */
+  if (m == 29) { d++; m = 1; } else  { m = nextwheel30[m]; }
   while (!_is_trial_prime7(d*30+m)) {
     m = nextwheel30[m];  if (m == 1) d++;
   }
@@ -189,8 +190,10 @@ UV next_prime(UV n)
 
   d = n/30;
   m = n - d*30;
-  m = nextwheel30[m];  if (m == 1) d++;
+  /* Move forward one, knowing we may not be on the wheel */
+  if (m == 29) { d++; m = 1; } else  { m = nextwheel30[m]; }
   while (!_is_prime7(d*30+m)) {
+    /* Move forward one, knowing we are on the wheel */
     m = nextwheel30[m];  if (m == 1) d++;
   }
   return(d*30+m);
@@ -785,10 +788,14 @@ UV nth_prime(UV n)
  *      by William H. Press et al.
  *
  * Any mistakes here are completely my fault.  This code has not been
- * verified for anything serious.  For better reulsts, see:
+ * verified for anything serious.  For better results, see:
  *    http://www.trnicely.net/pi/pix_0000.htm
  * which although the author claims are demonstration programs, will
- * produce more usable results than this code does.
+ * undoubtedly produce more reliable results than this code does (I don't
+ * know of any obvious issues with this code, but it just hasn't been used
+ * by many people).
+ *
+ * TODO: Verify error bounds at different ranges.
  */
 
 static double const euler_mascheroni = 0.57721566490153286060651209008240243104215933593992;
