@@ -23,10 +23,6 @@ extern double ExponentialIntegral(double x);
 extern double LogarithmicIntegral(double x);
 extern double RiemannR(double x);
 
-#define SEGMENT_CHUNK_SIZE  UVCONST(262144)
-extern unsigned char* get_prime_segment(void);
-extern void           free_prime_segment(void);
-
 /* Above this value, is_prime will do deterministic Miller-Rabin */
 /* With 64-bit math, we can do much faster mulmods from 2^16-2^32 */
 #if (BITS_PER_WORD == 64) || HAVE_STD_U64
